/*
 * QEMU Library functions
 *
 * Copyright (c) 2003-2005 Fabrice Bellard
 * Copyright (c) 2006 Dr. Juergen Pfennig (info@j-pfennig.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include "vl.h"

// used in qemu_tmp_file()
static int tmp_filename_idx = 0;

//------------------------------------------------------------------------------
// Call:        Get a temporary file name
//
// Params:      fileName    Buffer to receive the name
//              size        Number of chars in buffer (must be >= 5)
//              keepOpen    TRUE:  return a file handle ready for read/write
//                          FALSE: close the file, return value is not a handle
//
// Return:      handle (see open flag) -or- -1 on error
//------------------------------------------------------------------------------
int qemu_tmp_file(char* fileName, int size, BOOL keepOpen)
{
    if(fileName != NULL && size > 0)
         fileName[0] = 0;
    if(fileName == NULL || size < 5)
        return qemu_set_errno("qemu_tmp_file", EINVAL);

#ifdef _WIN32
    char* p = strrchr(fileName, '/');
    if (p == NULL)
        return qemu_set_errno("qemu_tmp_file", EINVAL);

    /* XXX: find a better function */
    tmpnam(p);
    *p = '/';
#else


// XXX: create something like qemu_tools.c to put trivial helpers there
// XXX: create qemu_info, qemu_message, qemu_error for report output
// XXX: qemu should have a --verbose option to enable qemu_info
// XXX: rename function to qemu_tmp_fileName() see also wfat.c

    const char* sTmp = getenv("TMPDIR");
    if(sTmp == 0 || sTmp[0] != '/' || access(sTmp, R_OK|W_OK|F_OK) < 0) {
        if(sTmp != 0) {
            unsetenv("TMPDIR");
            fprintf(stderr, "qemu: TMPDIR invalid, reverting to /tmp\n");
        }
        sTmp = "/tmp";
    }

    for(;;) {
        snprintf(fileName, size, "%s/qemu%d.%u", sTmp,
            getpid(), (unsigned int)tmp_filename_idx++);
        // file should not exist - ignore this one
        if(access(fileName, R_OK|W_OK|F_OK) == 0) continue;
        // XXX: will fail later, hope this message is helpfull
        if(errno != ENOENT)
            fprintf(stderr, "qemu: bad tmpfile: %s: %s\n",
                    fileName, strerror(errno));
//      else
//          fprintf(stderr, "qemu: using tmpfile: %s\n", fileName);
        break;
    }
#endif
    return 1;
}

//------------------------------------------------------------------------------
// Call:        returns the allocated file size in bytes
//------------------------------------------------------------------------------
#ifdef _WIN32
int64_t qemu_filesize(const char *filename)
{
    struct _stati64 st;
    if (_stati64(filename, &st) < 0)
        return -1;
    return st.st_size;
}
#else
int64_t qemu_filesize(const char *filename)
{
    struct stat st;
    if (stat(filename, &st) < 0)
        return -1;
    return (int64_t)st.st_blocks * 512;
}
#endif

//------------------------------------------------------------------------------
// Call:        Interrupt-safe seek() and read() function.
//
// Params:      file        .
//              buff
//              len
//              offs        -1  -or-  offset to start-of-file
//
// Return:      -1          on error, errno is set
//              0..len      number of bytes read
//------------------------------------------------------------------------------
int qemu_read_at(int file, void* buff, uint len, uint64_t offs)
{
    if(offs != QEMU_NO_SEEK && offs != lseek(file, offs, SEEK_SET))
        return -1;

    int cnt = len;
    while(cnt) {
        int ires = read(file, buff, cnt);
        if(ires > 0) {                              // got data
            cnt -= ires; buff = (uint8_t*)buff + ires;
        }
        else if(ires == 0)                          // EOF
            return len - cnt;
        else if(ires < 0 && errno != EINTR)         // IO error
            return -1;
    }
    return len;
}

//------------------------------------------------------------------------------
// Call:        Interrupt-safe seek() and write() function.
//
// Params:      file        .
//              buff
//              len
//              offs        -1  -or-  offset to start-of-file
//
// Return:      -1          on error, errno is set
//              len         number of bytes written
//------------------------------------------------------------------------------
int qemu_write_at(int file, const void* buff, uint len, uint64_t offs)
{
    if(offs != QEMU_NO_SEEK && offs != lseek(file, offs, SEEK_SET))
        return -1;

    int cnt = len;
    while(cnt) {
        int ires = write(file, buff, cnt);
        if(ires >= 0) {                             // wrote data
            cnt -= ires; buff = (uint8_t*)buff + ires;
        }
        else if(ires < 0 && errno != EINTR)         // IO error
            return -1;
    }
    return len;
}

//------------------------------------------------------------------------------
// Call:        Set the errno "variable" (errno is thread local storage)
//
// Params:      name        Name of the calling function
//              code        an explicit error number (overrides errno) -or- 0
//
// Return:      -1          (the most frequently used error return value)
//------------------------------------------------------------------------------
int qemu_set_errno(const char* name, int code)
{
    if(name == 0) name = "<?>";
    if(code <= 0) {
        if(errno == 0) errno = EINVAL;
    }
    else
        errno = code;
    perror(name);
    return -1;
}

//------------------------------------------------------------------------------
// Call:        Set the errno "variable" (errno is thread local storage)
//
// Params:      name        Name of the calling function
//              code        an explicit error number (overrides errno) -or- 0
//
// Return:      -1          (the most frequently used error return value)
//------------------------------------------------------------------------------
void qemu_progress(const char* prefix, const char* suffix, uint percent,
                   void (*prnt)(const char*, ...))
{
    const char* sHash = "#########################";    // 25 chars
    const char* sDots = ".........................";    // 25 chars
    const char* sForm = "\r%s%3d%% [%s%s] %s";

    if(percent > 100) percent = 100;
    prnt(sForm, prefix, percent, sHash+25-(percent/4), sDots+(percent/4), suffix);
}
